use serde::{Deserialize, Serialize};
use thiserror::Error;

use crate::identifier::{Identifier, IdentifierError};

// TODO: Needs to serialize/deserialize as the identifier with a hash in front.
#[derive(Debug, Serialize, Deserialize, Clone, Eq, PartialEq, Hash)]
#[serde(try_from = "&str", into = "String")]
pub struct Tag(Identifier);

impl Tag {
    pub fn new(namespace: String, path: String) -> Self {
        Self(Identifier::new(namespace, path))
    }

    pub fn minecraft(path: String) -> Self {
        Self::new("minecraft".to_string(), path)
    }

    pub fn new_str(namespace: &str, path: &str) -> Self {
        Self::new(namespace.to_string(), path.to_string())
    }

    pub fn minecraft_str(path: &str) -> Self {
        Self::minecraft(path.to_string())
    }
}

impl TryFrom<&str> for Tag {
    type Error = TagError;
    fn try_from(value: &str) -> Result<Self, Self::Error> {
        match value.strip_prefix("#") {
            Some(v) => Ok(Self(Identifier::try_from(v)?)),
            None => Err(TagError::MissingHash),
        }
    }
}

impl From<Tag> for String {
    fn from(value: Tag) -> Self {
        let v: String = value.0.into();
        format!("#{v}")
    }
}

#[derive(Debug, Error)]
pub enum TagError {
    #[error("Missing hash")]
    MissingHash,
    #[error("Invalid identifier: {0}")]
    InvalidIdentifier(#[from] IdentifierError),
}
